@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007-2018 S[&]T, The Netherlands.
+ * Copyright (C) 2007-2019 S[&]T, The Netherlands.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -544,7 +544,19 @@ int coda_path_for_program(const char *argv0, char **location)
 #ifdef WIN32
         int argv0_length = (int)strlen(argv0);
 
-        if (argv0_length <= 4 || strcmp(&argv0[argv0_length - 4], ".exe") != 0)
+        if (coda_path_find_file(".", argv0, location) != 0)
+        {
+            return -1;
+        }
+        if (*location == NULL && getenv("PATH") != NULL)
+        {
+            if (coda_path_find_file(getenv("PATH"), argv0, location) != 0)
+            {
+                return -1;
+            }
+        }
+
+        if (*location == NULL && (argv0_length <= 4 || strcmp(&argv0[argv0_length - 4], ".exe") != 0))
         {
             char *filepath;
 
@@ -572,20 +584,6 @@ int coda_path_for_program(const char *argv0, char **location)
             }
             free(filepath);
         }
-        else
-        {
-            if (coda_path_find_file(".", argv0, location) != 0)
-            {
-                return -1;
-            }
-            if (*location == NULL && getenv("PATH") != NULL)
-            {
-                if (coda_path_find_file(getenv("PATH"), argv0, location) != 0)
-                {
-                    return -1;
-                }
-            }
-        }
 #else
         if (getenv("PATH") != NULL)
         {
@@ -604,17 +602,17 @@ int coda_path_for_program(const char *argv0, char **location)
     if (*location != NULL && (*location)[0] != '/' && (*location)[0] != '\\' &&
         !(isalpha((*location)[0]) && (*location)[1] == ':'))
     {
-        char cwd[1024 + 1];
+        char cwd[4096 + 1];
         char *relative_location;
 
         /* change relative path into absolute path */
 
-        if (getcwd(cwd, 1024) == NULL)
+        if (getcwd(cwd, 4096) == NULL)
         {
             /* there is a problem with the current working directory -> return 'not found' */
             return 0;
         }
-        cwd[1024] = '\0';
+        cwd[4096] = '\0';
 
         relative_location = *location;
         if (coda_path_from_path(cwd, 0, relative_location, location) != 0)
