@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007-2018 S[&]T, The Netherlands.
+ * Copyright (C) 2007-2019 S[&]T, The Netherlands.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -60,7 +60,7 @@ void generate_detection_tree(coda_format format);
 static void print_version()
 {
     printf("codadd %s\n", libcoda_version);
-    printf("Copyright (C) 2007-2018 S[&]T, The Netherlands.\n");
+    printf("Copyright (C) 2007-2019 S[&]T, The Netherlands.\n");
     printf("\n");
 }
 
@@ -179,11 +179,8 @@ int main(int argc, char *argv[])
     }
     else
     {
-#ifdef WIN32
-        const char *definition_path = "../definitions";
-#else
         const char *definition_path = "../share/" PACKAGE "/definitions";
-#endif
+
         if (coda_set_definition_path_conditional(argv[0], NULL, definition_path) != 0)
         {
             fprintf(stderr, "ERROR: %s\n", coda_errno_to_string(coda_errno));
